# Generated by roxygen2: do not edit by hand

S3method("$",splines2)
S3method(deriv,BSpline)
S3method(deriv,BernsteinPoly)
S3method(deriv,CSpline)
S3method(deriv,ISpline)
S3method(deriv,MSpline)
S3method(deriv,NaturalSpline)
S3method(deriv,NaturalSplineK)
S3method(knots,splines2)
S3method(makepredictcall,BSpline)
S3method(makepredictcall,BernsteinPoly)
S3method(makepredictcall,CSpline)
S3method(makepredictcall,ISpline)
S3method(makepredictcall,MSpline)
S3method(makepredictcall,NaturalSpline)
S3method(makepredictcall,NaturalSplineK)
S3method(plot,splines2)
S3method(predict,BSpline)
S3method(predict,BernsteinPoly)
S3method(predict,CSpline)
S3method(predict,ISpline)
S3method(predict,MSpline)
S3method(predict,NaturalSpline)
S3method(predict,NaturalSplineK)
S3method(print,splines2)
S3method(update,BSpline)
S3method(update,BernsteinPoly)
S3method(update,CSpline)
S3method(update,ISpline)
S3method(update,MSpline)
S3method(update,NaturalSpline)
S3method(update,NaturalSplineK)
export(bSpline)
export(bernsteinPoly)
export(bpoly)
export(bsp)
export(cSpline)
export(csp)
export(dbs)
export(iSpline)
export(ibs)
export(isp)
export(mSpline)
export(msp)
export(naturalSpline)
export(nsk)
export(nsp)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,abline)
importFrom(graphics,matplot)
importFrom(stats,deriv)
importFrom(stats,knots)
importFrom(stats,makepredictcall)
importFrom(stats,predict)
importFrom(stats,update)
useDynLib(splines2)
